#include "NrsecPort.h"
#ifdef __ANDROID__
#include <android/log.h>
#endif

#define  RE_SUC   0x01
#define  RE_ERROR 0x00

#define TAG_SPI "SPI"

const uint8_t EK_CMD[5] = { 0x80,0xd4,0x01,0x00,0x10 };
const uint8_t AK_CMD[5] = { 0x80,0xd4,0x02,0x00,0x10 };
const uint8_t IV_CMD[5] = { 0x80,0xd4,0x04,0x00,0x10 };
uint8_t SM1Encrypt_CMD[5] = { 0xa0,0xe0,0x80,0xff,0xff };
uint8_t SM1decoder_CMD[5] = { 0xa0,0xe0,0x81,0xff,0xff };
uint8_t SM2Keypair_CMD[5] = { 0x80,0xb2,0x00,0xff,0x00 };
uint8_t SM2OutPub_CMD[5] = { 0x80,0xb8,0x01,0xff,0x40 };
uint8_t SM2OutPri_CMD[5] = { 0x80,0xb8,0x02,0xff,0x20 };
uint8_t SM2InPub_CMD[5] = { 0x80,0xba,0x01,0xff,0x40 };
uint8_t SM2InPri_CMD[5] = { 0x80,0xba,0x02,0xff,0x20 };
uint8_t SM3Hash_CMD[5] = { 0x80,0xb5,0x00,0xff,0xff };
uint8_t SM2Sign_CMD[5] = { 0x80,0xb4,0x00,0xff,0x20 };
uint8_t SM2VerifySign_CMD[5] = { 0x80,0xb6,0x00,0xff,0x60 };
uint8_t SM2encrypt_CMD[5] = { 0x80,0xb3,0x01,0xff,0x20 };
uint8_t SM2decoder_CMD[5] = { 0x80,0xb3,0x81,0xff,0x80 };
uint8_t SM2cert_CMD[5] = { 0x80,0xb7,0xff,0xff,0xff };
uint8_t Random_CMD[5] = { 0x00,0x84,0x00,0x00,0xff };
const    uint8_t Version_CMD[5] = { 0x00,0x5b,0x00,0x00,0x40 };
const    uint8_t Indentify_CMD[5] = { 0x80,0xb3,0x01,0x04,0x20 };

uint8_t NrsecPort::CalcCRC7(const uint8_t *buff, int len)
{
	const static uint8_t crc7_table[256] = {
			0x00, 0x09, 0x12, 0x1b, 0x24, 0x2d, 0x36, 0x3f,
			0x48, 0x41, 0x5a, 0x53, 0x6c, 0x65, 0x7e, 0x77,
			0x19, 0x10, 0x0b, 0x02, 0x3d, 0x34, 0x2f, 0x26,
			0x51, 0x58, 0x43, 0x4a, 0x75, 0x7c, 0x67, 0x6e,
			0x32, 0x3b, 0x20, 0x29, 0x16, 0x1f, 0x04, 0x0d,
			0x7a, 0x73, 0x68, 0x61, 0x5e, 0x57, 0x4c, 0x45,
			0x2b, 0x22, 0x39, 0x30, 0x0f, 0x06, 0x1d, 0x14,
			0x63, 0x6a, 0x71, 0x78, 0x47, 0x4e, 0x55, 0x5c,
			0x64, 0x6d, 0x76, 0x7f, 0x40, 0x49, 0x52, 0x5b,
			0x2c, 0x25, 0x3e, 0x37, 0x08, 0x01, 0x1a, 0x13,
			0x7d, 0x74, 0x6f, 0x66, 0x59, 0x50, 0x4b, 0x42,
			0x35, 0x3c, 0x27, 0x2e, 0x11, 0x18, 0x03, 0x0a,
			0x56, 0x5f, 0x44, 0x4d, 0x72, 0x7b, 0x60, 0x69,
			0x1e, 0x17, 0x0c, 0x05, 0x3a, 0x33, 0x28, 0x21,
			0x4f, 0x46, 0x5d, 0x54, 0x6b, 0x62, 0x79, 0x70,
			0x07, 0x0e, 0x15, 0x1c, 0x23, 0x2a, 0x31, 0x38,
			0x41, 0x48, 0x53, 0x5a, 0x65, 0x6c, 0x77, 0x7e,
			0x09, 0x00, 0x1b, 0x12, 0x2d, 0x24, 0x3f, 0x36,
			0x58, 0x51, 0x4a, 0x43, 0x7c, 0x75, 0x6e, 0x67,
			0x10, 0x19, 0x02, 0x0b, 0x34, 0x3d, 0x26, 0x2f,
			0x73, 0x7a, 0x61, 0x68, 0x57, 0x5e, 0x45, 0x4c,
			0x3b, 0x32, 0x29, 0x20, 0x1f, 0x16, 0x0d, 0x04,
			0x6a, 0x63, 0x78, 0x71, 0x4e, 0x47, 0x5c, 0x55,
			0x22, 0x2b, 0x30, 0x39, 0x06, 0x0f, 0x14, 0x1d,
			0x25, 0x2c, 0x37, 0x3e, 0x01, 0x08, 0x13, 0x1a,
			0x6d, 0x64, 0x7f, 0x76, 0x49, 0x40, 0x5b, 0x52,
			0x3c, 0x35, 0x2e, 0x27, 0x18, 0x11, 0x0a, 0x03,
			0x74, 0x7d, 0x66, 0x6f, 0x50, 0x59, 0x42, 0x4b,
			0x17, 0x1e, 0x05, 0x0c, 0x33, 0x3a, 0x21, 0x28,
			0x5f, 0x56, 0x4d, 0x44, 0x7b, 0x72, 0x69, 0x60,
			0x0e, 0x07, 0x1c, 0x15, 0x2a, 0x23, 0x38, 0x31,
			0x46, 0x4f, 0x54, 0x5d, 0x62, 0x6b, 0x70, 0x79
	};

	uint8_t crc7_accum = 0;
	int i;

	for (i = 0; i < len; i++) {
		crc7_accum =
			crc7_table[(crc7_accum << 1) ^ buff[i]];
	}
	return crc7_accum;
}

void NrsecPort::SendCMD(uint8_t* cmd, uint8_t* rxbuf)
{
	int i = 0;
	int retval;

#if defined (CONFIG_ATMEL_SPI_DEBUG)
	printf("tx %1d bytes: ", CMD_HEAD_SIZE);
	for (i = 0; i < CMD_HEAD_SIZE; i++)
	{
		printf(" %02x", cmd[i]);
	}
	printf("\n");
#endif

	/* send five command header */
	for (i = 0; i < CMD_HEAD_SIZE; i++)
	{
		retval = spi_transfer(cmd + i, rxbuf + i, 1);
#ifdef __ANDROID__
		__android_log_print(ANDROID_LOG_INFO, TAG_SPI, "cmd[%d]=%x,rxbuf[%d]=%x", i, *(cmd + i), i, *(rxbuf + i));
#endif
		delay(20);
	}

	cmd[0] = 0xaa;    //for response
}

void NrsecPort::RcvINS(uint8_t* txbuf, uint8_t* buf, uint8_t ins)
{
	int retval;
	int cnt = 1000;
	int count = 0;
	/* receive ins */
INS:
	txbuf[0] = 0xaa;
	delay(20);
	while (cnt--)
	{
		retval = spi_transfer(txbuf, buf, 1);
#ifdef __ANDROID__
		__android_log_print(ANDROID_LOG_INFO, TAG_SPI, "RcvINS txbuf=%x,buf=%x", *txbuf, *buf);
#endif
		if (*buf == ins)
		{
			return;
			break;
		}
		else
			goto INS;
	}
}

void NrsecPort::RcvLEN(uint8_t* txbuf, uint8_t* buf, uint8_t len)
{
	int retval;
	/* receive length */
LEN:
	for (int i = 0; i < len; i++) {
		txbuf[0] = 0xaa;
		retval = spi_transfer(txbuf, buf + i, 1);
#ifdef __ANDROID__
		__android_log_print(ANDROID_LOG_INFO, TAG_SPI, "RecvLEN txbuf=%x,rxbuf=%x", *txbuf, *(buf + i));
#endif
	}
}

//Rcvdata
void NrsecPort::RcvData(uint8_t*txbuf, uint8_t* buf)
{
	int len = *(buf - 1);
	RcvData(txbuf, buf, len);
}

void NrsecPort::RcvData(uint8_t*txbuf, uint8_t*buf, int len)
{
	int i;
	int retval;

	for (i = 0; i < len; i++)
	{
		*(txbuf + i) = 0xaa;
	}

	/* receive data and crc */
	for (i = 0; i < len; i++)
	{
		retval = spi_transfer(txbuf, buf + i, 1);
		// __android_log_print(ANDROID_LOG_INFO, TAG_SPI, "RcvData data[%d]=%x", i, *(buf + i));
	}
}

//RcvSW
void NrsecPort::RcvSW(uint8_t*txbuf, uint8_t*buf, uint8_t sw)
{
	int i;
	int retval;

SW90:
	/* receive state word */
	delay(20);
	while (1)
	{
		retval = spi_transfer(txbuf, buf, 1);
#ifdef __ANDROID__
		__android_log_print(ANDROID_LOG_INFO, TAG_SPI, "RecvSW txbuf=%x,buf=%x", *txbuf, *buf);
#endif
		if (*buf != sw)
		{
			goto SW90;
		}
		break;
	}
	retval = spi_transfer(txbuf, buf + 1, 1);
}

void NrsecPort::SendEnd(uint8_t* txbuf, uint8_t* buf)
{
	int retval;
	txbuf[0] = 0xaa;
	retval = spi_transfer(txbuf, buf, 1);
#ifdef __ANDROID__
	__android_log_print(ANDROID_LOG_INFO, TAG_SPI, "SendEnd txbuf=%x,rxbuf=%hhu", *txbuf, *buf);
#endif
}

void NrsecPort::SendId(uint8_t* txbuf, uint8_t* buf, uint8_t id)
{
	int retval;
	txbuf[0] = id;
	retval = spi_transfer(txbuf, buf, 1);
#ifdef __ANDROID__
	__android_log_print(ANDROID_LOG_INFO, TAG_SPI, "SendID txbuf=%x,rxbuf=%hhu", *txbuf, *buf);
#endif
}

void NrsecPort::SendData(uint8_t* data, uint8_t* rxbuf, int data_size)
{
	int i = 0;
	int retval;
	uint8_t crc[1];
	crc[0] = CalcCRC7(data, data_size);

	for (i = 0; i < data_size; i++)
	{
		retval = spi_transfer(data + i, rxbuf + i, 1);
#ifdef __ANDROID__
		__android_log_print(ANDROID_LOG_INFO, TAG_SPI, "SendData i=%d,txbuf=%x,rxbuf=%x", i, *(data + i), *(rxbuf + i));
#endif
		delay(20);
	}
	retval = spi_transfer(crc, rxbuf, 1);
}

int NrsecPort::SM1ImportKey(const uint8_t* ek, const uint8_t* ak)
{
    int i;
    int cnt;
    uint8_t txbuf[256];
    uint8_t rxbuf[256];

    int retval;
    int msglen;

    cnt = 0;
    msglen = 5;
    memset(rxbuf, 0, sizeof(rxbuf));
    memset(txbuf, 0, sizeof(txbuf));

    CMD_RESEND:

    memcpy(txbuf, (const void *)EK_CMD, sizeof(EK_CMD));

    SendCMD(txbuf, rxbuf);

    RcvINS(txbuf, rxbuf, txbuf[1]);     // 指令

    SendId(txbuf, rxbuf, 0x55);

    memcpy(txbuf, ek, 16);
//    txbuf[16] = CalcCRC7(ek, 16);

    SendData(txbuf, rxbuf, 16);

    RcvSW(txbuf, rxbuf, 0x90);

	memcpy(txbuf, (const void *)AK_CMD, sizeof(AK_CMD));

	SendCMD(txbuf, rxbuf);

	RcvINS(txbuf, rxbuf, txbuf[1]);     // 指令

	SendId(txbuf, rxbuf, 0x55);

	memcpy(txbuf, ak, 16);
//	txbuf[16] = CalcCRC7(ak, 16);

	SendData(txbuf, rxbuf, 16);//senddata函数中已经包含crc了

	SendEnd(txbuf, rxbuf);

	RcvSW(txbuf, rxbuf, 0x90);

    return 0;
}

int NrsecPort::ImportIV(const uint8_t* iv, uint8_t ivLength)
{
	int i;
	int cnt;
	uint8_t txbuf[256];
	uint8_t rxbuf[256];

	int retval;
	int msglen;

	cnt = 0;
	msglen = 5;
	memset(rxbuf, 0, sizeof(rxbuf));
	memset(txbuf, 0, sizeof(txbuf));

CMD_RESEND:

	memcpy(txbuf, (const void *)IV_CMD, sizeof(IV_CMD));

	SendCMD(txbuf, rxbuf);

	RcvINS(txbuf, rxbuf, txbuf[1]);     // 指令

	SendId(txbuf, rxbuf, 0x55);

	memcpy(txbuf, iv, ivLength);
//	txbuf[ivLength] = CalcCRC7(iv, ivLength);

	SendData(txbuf, rxbuf, ivLength);

	SendEnd(txbuf, rxbuf);

	RcvSW(txbuf, rxbuf, 0x90);

	return 0;
}

int NrsecPort::SM1Encrypt(const uint8_t* data, uint16_t dataLen, uint8_t* encryptedData, uint16_t bufferLen)
{
    int i;
    int cnt;
    uint8_t txbuf[256];
    uint8_t rxbuf[256];

    int retval;
    int msglen;

    cnt = 0;
    msglen = 5;
    memset(rxbuf, 0, sizeof(rxbuf));
    memset(txbuf, 0, sizeof(txbuf));

    //printf("tx %1d bytes: ", msglen);

    CMD_RESEND:

    memcpy(txbuf, (const void *)SM1Encrypt_CMD, sizeof(SM1Encrypt_CMD));

    txbuf[3] = dataLen >> 8;
    txbuf[4] = dataLen & 0xFF;

    SendCMD(txbuf, rxbuf);

    RcvINS(txbuf, rxbuf, txbuf[1]);     // 指令

    SendId(txbuf, rxbuf, 0x55);

	memcpy(txbuf,data,dataLen);

	SendData(txbuf,rxbuf,dataLen);

	SendEnd(txbuf,rxbuf);

	RcvINS(txbuf,rxbuf,0xe0);

    RcvLEN(txbuf, rxbuf + 1, 2);  //长度 多加一个字节的 CRC

//    uint8_t len = *(rxbuf + 1);
	uint8_t len =(*(rxbuf+1)<<8)+(*(rxbuf+2));

	RcvData(txbuf, rxbuf + 3, len);

    RcvSW(txbuf, rxbuf + 3 + len, 0x90);

    //计算接收到数据的CRC
    if (CalcCRC7(rxbuf + 3, len-1) != rxbuf[len + 1])
    {
        //CRC Error  命令重发，超过3次，结束
        if (cnt < 3)
        {
            cnt++;
            goto CMD_RESEND;
            printf("cnt over\n");
        }
        else
        {
            printf("ERROR\n");
        }
    }

    // printf("rx %1d bytes: ", rxbuf[1] + 4);
#ifdef __ANDROID__
    __android_log_print(ANDROID_LOG_INFO, TAG_SPI, "rx %1d bytes:", rxbuf[1] + 4);
#endif

    memcpy(encryptedData, rxbuf + 3, len-1);
	memcpy(&bufferLen, &len-1, 1);
    return 0;
}

int NrsecPort::SM1Decrypt(const uint8_t* encryptedData, uint16_t encryptedDataLen, uint8_t* data, uint16_t bufferLen)
{
	int i;
	int cnt;
	uint8_t txbuf[256];
	uint8_t rxbuf[256];

	int retval;
	int msglen;

	cnt = 0;
	msglen = 5;
	memset(rxbuf, 0, sizeof(rxbuf));
	memset(txbuf, 0, sizeof(txbuf));

	//printf("tx %1d bytes: ", msglen);

	CMD_RESEND:

	memcpy(txbuf, (const void *)SM1decoder_CMD, sizeof(SM1decoder_CMD));

	txbuf[3] = encryptedDataLen >> 8;
	txbuf[4] = encryptedDataLen & 0xFF;

	SendCMD(txbuf, rxbuf);

	RcvINS(txbuf, rxbuf, txbuf[1]);     // 指令

	SendId(txbuf, rxbuf, 0x55);

	memcpy(txbuf,encryptedData,encryptedDataLen);

	SendData(txbuf,rxbuf,encryptedDataLen);

	SendEnd(txbuf,rxbuf);

	RcvINS(txbuf,rxbuf,0xe0);

	RcvLEN(txbuf, rxbuf + 1, 2);  //长度 多加一个字节的 CRC

//    uint8_t len = *(rxbuf + 1);
	uint8_t len =(*(rxbuf+1)<<8)+(*(rxbuf+2));

	RcvData(txbuf, rxbuf + 3, len);

	RcvSW(txbuf, rxbuf + 3 + len, 0x90);

	//计算接收到数据的CRC
	if (CalcCRC7(rxbuf + 3, len-1) != rxbuf[len + 1])
	{
		//CRC Error  命令重发，超过3次，结束
		if (cnt < 3)
		{
			cnt++;
			goto CMD_RESEND;
			printf("cnt over\n");
		}
		else
		{
			printf("ERROR\n");
		}
	}

	// printf("rx %1d bytes: ", rxbuf[1] + 4);
#ifdef __ANDROID__
	__android_log_print(ANDROID_LOG_INFO, TAG_SPI, "rx %1d bytes:", rxbuf[1] + 4);
#endif

	memcpy(data, rxbuf + 3, len-1);
	memcpy(&bufferLen, &len-1, 1);
	return 0;

}

int NrsecPort::Random(uint8_t* output, uint8_t length)
{
	int i;
	int cnt;
	uint8_t txbuf[256];
	uint8_t rxbuf[256];

	int retval;
	int msglen;

	cnt = 0;
	msglen = 5;
	memset(rxbuf, 0, sizeof(rxbuf));
	memset(txbuf, 0, sizeof(txbuf));

	//printf("tx %1d bytes: ", msglen);

CMD_RESEND:

	memcpy(txbuf, (const void *)Random_CMD, sizeof(Random_CMD));

	txbuf[4] = length;

	SendCMD(txbuf, rxbuf);

	RcvINS(txbuf, rxbuf, txbuf[1]);     // 指令

	RcvLEN(txbuf, rxbuf + 1, 1);  //长度 多加一个字节的 CRC
	
	uint8_t len = *(rxbuf + 1);

	RcvData(txbuf, rxbuf + 2, len);

	RcvSW(txbuf, rxbuf + 2 + len, 0x90);

	//计算接收到数据的CRC
	if (CalcCRC7(rxbuf + 2, rxbuf[1] - 1) != rxbuf[rxbuf[1] + 1])
	{
		//CRC Error  命令重发，超过3次，结束
		if (cnt < 3)
		{
			cnt++;
			goto CMD_RESEND;
			printf("cnt over\n");
		}
		else
		{
			printf("ERROR\n");
		}
	}

	// printf("rx %1d bytes: ", rxbuf[1] + 4);
#ifdef __ANDROID__
	__android_log_print(ANDROID_LOG_INFO, TAG_SPI, "rx %1d bytes:", rxbuf[1] + 4);
#endif

	memcpy(output, rxbuf + 2, len-1);
	
	return 0;
}

std::string NrsecPort::Version()
{
	int i;
	int cnt;
	uint8_t txbuf[256];
	uint8_t rxbuf[256];

	int retval;
	int msglen;

	cnt = 0;
	msglen = 5;
	memset(rxbuf, 0, sizeof(rxbuf));
	memset(txbuf, 0, sizeof(txbuf));

	//printf("tx %1d bytes: ", msglen);
	//__android_log_print(ANDROID_LOG_INFO, TAG_SPI, "tx %1d bytes", msglen);

CMD_RESEND:

	memcpy(txbuf, Version_CMD, sizeof(Version_CMD));

	SendCMD(txbuf, rxbuf);

	RcvINS(txbuf, rxbuf, txbuf[1]);     // 指令

	// txbuf[4] = 0xAA;
	RcvLEN(txbuf, rxbuf + 1, 1);  //长度 多加一个字节的 CRC

	int dataLen = *(rxbuf + 1);

	RcvData(txbuf, rxbuf + 2, dataLen);

	RcvSW(txbuf, rxbuf + 2 + rxbuf[1], 0x90);

	//计算接收到数据的CRC
	if (CalcCRC7(rxbuf + 2, rxbuf[1] - 1) != rxbuf[rxbuf[1] + 1])
	{
		//CRC Error  命令重发，超过3次，结束
		if (cnt < 3)
		{
			cnt++;
			goto CMD_RESEND;
			printf("cnt over\n");
		}
		else
		{
			printf("ERROR\n");
		}
	}

	//printf("rx %1d bytes: ", rxbuf[1]+4);
#ifdef __ANDROID__
	__android_log_print(ANDROID_LOG_INFO, TAG_SPI, "rx %1d bytes:", rxbuf[1] + 4);
#endif
	std::string version = "";
	char output[16] = { 0 };
	for (i = 0; i < dataLen-1; i++) {
		if (*(rxbuf + 2 + i) == 0) {
			break;
		}
		snprintf(output, sizeof(output), "%c", *(rxbuf + 2 + i));
		version += output;
	}
#ifdef __ANDROID__
	__android_log_print(ANDROID_LOG_INFO, TAG_SPI, "Version: %s", version.c_str());
#endif
	//__android_log_print(ANDROID_LOG_INFO, "SPi", "%s", rxbuf);
	// printf("\n");


	return version;
}

int NrsecPort::SM2keypair(int index)//产生密钥
{
	int i;
	int cnt;
	uint8_t txbuf[256];
	uint8_t rxbuf[256];

	int retval;
	int msglen;

	msglen = 5;
	memset(rxbuf, 0, sizeof(rxbuf));
	memset(txbuf, 0, sizeof(txbuf));

	SM2Keypair_CMD[3] = index;

	//printf("tx %1d bytes: ", msglen);
	//__android_log_print(ANDROID_LOG_INFO, TAG_SPI, "tx %1d bytes", msglen);

CMD_RESEND:

	memcpy(txbuf, (const void *)SM2Keypair_CMD, sizeof(SM2Keypair_CMD));

	SendCMD(txbuf, rxbuf);

	RcvSW(txbuf, rxbuf, 0x90);



	return 0;
}

int NrsecPort::SM2ExportPublicKey(int index, uint8_t result[])//导出公钥
{
	int i;
	int cnt;
	uint8_t txbuf[256];
	uint8_t rxbuf[256];

	int retval;
	int msglen;

	cnt = 0;
	msglen = 5;
	memset(rxbuf, 0, sizeof(rxbuf));
	memset(txbuf, 0, sizeof(txbuf));

	SM2OutPub_CMD[3] = index;


CMD_RESEND:

	memcpy(txbuf, (const void *)SM2OutPub_CMD, sizeof(SM2OutPub_CMD));

	SendCMD(txbuf, rxbuf);

	RcvINS(txbuf, rxbuf, txbuf[1]);     // 指令

	RcvLEN(txbuf, rxbuf + 1, txbuf[4]);  //长度 多加一个字节的 CRC

	RcvData(txbuf, rxbuf + 2);

	RcvSW(txbuf, rxbuf + 2 + rxbuf[1], 0x90);

	//计算接收到数据的CRC
	if (CalcCRC7(rxbuf + 2, rxbuf[1] - 1) != rxbuf[rxbuf[1] + 1])
	{
		//CRC Error  命令重发，超过3次，结束
		if (cnt < 3)
		{
			cnt++;
			goto CMD_RESEND;
			printf("cnt over\n");
		}
		else
		{
			printf("ERROR\n");
		}
	}

	//printf("rx %1d bytes: ", rxbuf[1]+4);
#ifdef __ANDROID__
	__android_log_print(ANDROID_LOG_INFO, TAG_SPI, "rx %1d bytes:", rxbuf[1] + 4);
#endif
	memcpy(result, rxbuf + 2, rxbuf[1]-1);
	return 0;
}

int NrsecPort::SM2ExportPrivateKey(int index, uint8_t result[])//导出私钥
{
	int i;
	int cnt;
	uint8_t txbuf[256];
	uint8_t rxbuf[256];

	int retval;
	int msglen;

	cnt = 0;
	msglen = 5;
	memset(rxbuf, 0, sizeof(rxbuf));
	memset(txbuf, 0, sizeof(txbuf));

	SM2OutPri_CMD[3] = index;

	//printf("tx %1d bytes: ", msglen);
	//__android_log_print(ANDROID_LOG_INFO, TAG_SPI, "tx %1d bytes", msglen);

CMD_RESEND:

	memcpy(txbuf, (const void *)SM2OutPri_CMD, sizeof(SM2OutPri_CMD));

	SendCMD(txbuf, rxbuf);

	RcvINS(txbuf, rxbuf, txbuf[1]);     // 指令

	RcvLEN(txbuf, rxbuf + 1, txbuf[4]);  //长度 多加一个字节的 CRC

	RcvData(txbuf, rxbuf + 2);

	RcvSW(txbuf, rxbuf + 2 + rxbuf[1], 0x90);

	//计算接收到数据的CRC
	if (CalcCRC7(rxbuf + 2, rxbuf[1] - 1) != rxbuf[rxbuf[1] + 1])
	{
		//CRC Error  命令重发，超过3次，结束
		if (cnt < 3)
		{
			cnt++;
			goto CMD_RESEND;
			printf("cnt over\n");
		}
		else
		{
			printf("ERROR\n");
		}
	}

	//printf("rx %1d bytes: ", rxbuf[1]+4);
#ifdef __ANDROID__
	__android_log_print(ANDROID_LOG_INFO, TAG_SPI, "rx %1d bytes:", rxbuf[1] + 4);
#endif

	memcpy(result, rxbuf + 2, rxbuf[1]-1);

	return 0;
}

int NrsecPort::SM2ImportPublicKey(int index, const uint8_t new_key[])//外部公钥导入存放在01
{
	int i;
	int cnt;
	uint8_t txbuf[256];
	uint8_t rxbuf[256];

	int retval;
	int msglen;

	msglen = 5;
	memset(rxbuf, 0, sizeof(rxbuf));
	memset(txbuf, 0, sizeof(txbuf));

	SM2InPub_CMD[3] = index;

	//printf("tx %1d bytes: ", msglen);
	//__android_log_print(ANDROID_LOG_INFO, "SPi", "tx %1d bytes", msglen);

CMD_RESEND:

	memcpy(txbuf, (const void *)SM2InPub_CMD, sizeof(SM2InPub_CMD));

	SendCMD(txbuf, rxbuf);

	RcvINS(txbuf, rxbuf, txbuf[1]);     // 指令

	SendId(txbuf, rxbuf, 0x55);

	memcpy(txbuf, new_key, 64);

	SendData(txbuf, rxbuf, 64);

	SendEnd(txbuf, rxbuf);

	RcvSW(txbuf, rxbuf + 1, 0x90);

	std::string result = "InPub: success";
#ifdef __ANDROID__
	__android_log_print(ANDROID_LOG_INFO, TAG_SPI, "%s", result.c_str());
#endif
	//__android_log_print(ANDROID_LOG_INFO, "SPi", "%s", rxbuf);
	// printf("\n");


	return 0;
}

int NrsecPort::SM2ImportPrivateKey(int index, const uint8_t new_key[])//导入私钥 没测试
{
	int i;
	int cnt;
	uint8_t txbuf[256];
	uint8_t rxbuf[256];

	int retval;
	int msglen;

	msglen = 5;
	memset(rxbuf, 0, sizeof(rxbuf));
	memset(txbuf, 0, sizeof(txbuf));

	SM2InPri_CMD[3] = index;

	//printf("tx %1d bytes: ", msglen);
	//__android_log_print(ANDROID_LOG_INFO, "SPi", "tx %1d bytes", msglen);

CMD_RESEND:

	memcpy(txbuf, (const void *)SM2InPri_CMD, sizeof(SM2InPri_CMD));

	SendCMD(txbuf, rxbuf);

	RcvINS(txbuf, rxbuf, txbuf[1]);     // 指令

	SendId(txbuf, rxbuf, 0x55);

	memcpy(txbuf, new_key, 32);

	SendData(txbuf, rxbuf, 32);

	SendEnd(txbuf, rxbuf);

	RcvSW(txbuf, rxbuf + 1, 0x90);

	std::string result = "InPri: success";
#ifdef __ANDROID__
	__android_log_print(ANDROID_LOG_INFO, TAG_SPI, "%s", result.c_str());
#endif


	return 0;
}

int NrsecPort::SM3Hash(uint8_t *to_hash, int len, uint8_t *out_hash)//原始哈希 跑通了，没有验证
{
	int i;
	int cnt;
	uint8_t txbuf[512];
	uint8_t rxbuf[512];

	int retval;
	int msglen;

	msglen = 5;
	memset(rxbuf, 0, sizeof(rxbuf));
	memset(txbuf, 0, sizeof(txbuf));


	SM3Hash_CMD[3] = len >> 8;
	SM3Hash_CMD[4] = len & 0xFF;

#ifdef __ANDROID__
	__android_log_print(ANDROID_LOG_INFO, "len", "len=%x", len);
#endif
	//printf("tx %1d bytes: ", msglen);
	//__android_log_print(ANDROID_LOG_INFO, "SPi", "tx %1d bytes", msglen);

CMD_RESEND:

	memcpy(txbuf, (const void *)SM3Hash_CMD, sizeof(SM3Hash_CMD));

	SendCMD(txbuf, rxbuf);

	RcvINS(txbuf, rxbuf, txbuf[1]);

	SendId(txbuf, rxbuf, 0x55);

	memcpy(txbuf, to_hash, len);

	SendData(txbuf, rxbuf, len);

	SendEnd(txbuf, rxbuf);

	memcpy(txbuf, (const void *)SM3Hash_CMD, sizeof(SM3Hash_CMD));

	RcvINS(txbuf, rxbuf, txbuf[1]);

	RcvLEN(txbuf, rxbuf + 1, 1);

	RcvData(txbuf, rxbuf + 2);

	RcvSW(txbuf, rxbuf + 2 + rxbuf[1], 0x90);

	//__android_log_print(ANDROID_LOG_INFO, "SPi", "rx %1d bytes:", rxbuf[1]+4);

	memcpy(out_hash, rxbuf + 2, rxbuf[1]-1);

	return 0;
}

int NrsecPort::SM3Hash(uint8_t *in, int inl, uint8_t *out, uint8_t *pubkey, uint8_t* pucID, int idl)
{
	int nRet, l;
	uint8_t *Z = NULL;
	int entl = 0;
	uint8_t tmpm[32];
	uint8_t abxy[32 * 4] = {
			0xFF,0xFF,0xFF,0xFE,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, /* a */
			0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
			0x00,0x00,0x00,0x00,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
			0xFF,0xFC,
			0x28,0xE9,0xFA,0x9E,0x9D,0x9F,0x5E,0x34,0x4D,0x5A,/* b */
			0x9E,0x4B,0xCF,0x65,0x09,0xA7,0xF3,0x97,0x89,0xF5,
			0x15,0xAB,0x8F,0x92,0xDD,0xBC,0xBD,0x41,0x4D,0x94,
			0x0E,0x93,
			0x32,0xC4,0xAE,0x2C,0x1F,0x19,0x81,0x19,0x5F,0x99,/* x */
			0x04,0x46,0x6A,0x39,0xC9,0x94,0x8F,0xE3,0x0B,0xBF,
			0xF2,0x66,0x0B,0xE1,0x71,0x5A,0x45,0x89,0x33,0x4C,
			0x74,0xC7,
			0xBC,0x37,0x36,0xA2,0xF4,0xF6,0x77,0x9C,0x59,0xBD,/* y */
			0xCE,0xE3,0x6B,0x69,0x21,0x53,0xD0,0xA9,0x87,0x7C,
			0xC6,0x2A,0x47,0x40,0x02,0xDF,0x32,0xE5,0x21,0x39,
			0xF0,0xA0
	};
	l = 2 + idl + 32 * 6;
	Z = (uint8_t *)malloc(l);
	if (!Z)
		return -1;
	entl = idl * 8;
	memset(Z + 1, entl & 0xFF, 1);
	entl >>= 8;
	memset(Z, entl & 0xFF, 1);
	memcpy(Z + 2, pucID, idl);
	memcpy(Z + 2 + idl, abxy, 32 * 4);
	memcpy(Z + 2 + idl + 4 * 32, pubkey, 32);
	memcpy(Z + 2 + idl + 5 * 32, pubkey + 32, 32);
	nRet = SM3Hash(Z, l, tmpm);
	if (nRet != 0)
		goto quit;
	free(Z);
	l = inl + 32;
	Z = (uint8_t *)malloc(l);
	if (!Z) {
		nRet = -1;
		goto quit;
	}
	memcpy(Z, tmpm, 32);
	memcpy(Z + 32, in, inl);
	nRet = SM3Hash(Z, l, out);
quit:
	if (Z)
		free(Z);
	return nRet;
}//用于签名的处理哈希值,用户标识01*16

int NrsecPort::SM2Sign(int index, const uint8_t *to_sign, uint8_t *out_sign)//SM2签名 所使用的哈希值应该来源于sm3hash_tosm2
{
	int i;
	int cnt;
	uint8_t txbuf[256];
	uint8_t rxbuf[256];

	int retval;
	int msglen;

	msglen = 5;
	memset(rxbuf, 0, sizeof(rxbuf));
	memset(txbuf, 0, sizeof(txbuf));

	//printf("tx %1d bytes: ", msglen);
	//__android_log_print(ANDROID_LOG_INFO, "SPi", "tx %1d bytes", msglen);
	SM2Sign_CMD[3] = index;

CMD_RESEND:

	memcpy(txbuf, (const void *)SM2Sign_CMD, sizeof(SM2Sign_CMD));

	SendCMD(txbuf, rxbuf);

	RcvINS(txbuf, rxbuf, txbuf[1]);

	SendId(txbuf, rxbuf, 0x55);

	memcpy(txbuf, to_sign, 32);

	SendData(txbuf, rxbuf, 32);

	SendEnd(txbuf, rxbuf);

	memcpy(txbuf, (const void *)SM2Sign_CMD, sizeof(SM2Sign_CMD));

	RcvINS(txbuf, rxbuf, txbuf[1]);

	RcvLEN(txbuf, rxbuf + 1, 1);

	RcvData(txbuf, rxbuf + 2);

	RcvSW(txbuf, rxbuf + 2 + rxbuf[1], 0x90);
#ifdef __ANDROID__
	__android_log_print(ANDROID_LOG_INFO, TAG_SPI, "rx %1d bytes:", rxbuf[1] + 4);
#endif
	memcpy(out_sign, rxbuf + 2, rxbuf[1]-1);
	return 0;
}

int NrsecPort::SM2VerifySign(int index, uint8_t *hash, uint8_t *vs)//SM2验签
{
	uint8_t txbuf[256];
	uint8_t rxbuf[256];

	int retval;
	int msglen;

	msglen = 5;
	memset(rxbuf, 0, sizeof(rxbuf));
	memset(txbuf, 0, sizeof(txbuf));

	//printf("tx %1d bytes: ", msglen);
	//__android_log_print(ANDROID_LOG_INFO, "SPi", "tx %1d bytes", msglen);
	SM2VerifySign_CMD[3] = index;

CMD_RESEND:

	memcpy(txbuf, (const void *)SM2VerifySign_CMD, sizeof(SM2VerifySign_CMD));

	SendCMD(txbuf, rxbuf);

	RcvINS(txbuf, rxbuf, txbuf[1]);

	SendId(txbuf, rxbuf, 0x55);

	memcpy(txbuf, hash, 32);
	memcpy(txbuf + 32, vs, 64);

	SendData(txbuf, rxbuf, 96);

	SendEnd(txbuf, rxbuf);

	RcvSW(txbuf, rxbuf, 0x90);
#ifdef __ANDROID__
	__android_log_print(ANDROID_LOG_INFO, "SPi", "rx %1d bytes:", rxbuf[1] + 4);
#endif

	return 0;
}

int NrsecPort::SM2Encrypt(int index, uint8_t *to_encrypt, uint8_t *out_encrypt)//加密
{
	int i;
	int cnt;
	uint8_t txbuf[512];
	uint8_t rxbuf[512];

	int retval;
	int msglen;

	msglen = 5;
	memset(rxbuf, 0, sizeof(rxbuf));
	memset(txbuf, 0, sizeof(txbuf));

	//printf("tx %1d bytes: ", msglen);
	//__android_log_print(ANDROID_LOG_INFO, "SPi", "tx %1d bytes", msglen);
	SM2encrypt_CMD[3] = index;

CMD_RESEND:

	memcpy(txbuf, (const void *)SM2encrypt_CMD, sizeof(SM2encrypt_CMD));

	SendCMD(txbuf, rxbuf);

	RcvINS(txbuf, rxbuf, txbuf[1]);

	SendId(txbuf, rxbuf, 0x55);

	memcpy(txbuf, to_encrypt, 32);

	SendData(txbuf, rxbuf, 32);

	SendEnd(txbuf, rxbuf);

	memcpy(txbuf, (const void *)SM2encrypt_CMD, sizeof(SM2encrypt_CMD));

	RcvINS(txbuf, rxbuf, txbuf[1]);

	RcvLEN(txbuf, rxbuf + 1, 1);

	RcvData(txbuf, rxbuf + 2);

	RcvSW(txbuf, rxbuf + 2 + rxbuf[1], 0x90);

	//__android_log_print(ANDROID_LOG_INFO, "SPi", "rx %1d bytes:", rxbuf[1]+4);
	memcpy(out_encrypt, rxbuf + 2, rxbuf[1]-1);

	return 0;
}
int NrsecPort::SM2Decrypt(int index, uint8_t *to_decoder, uint8_t *out_decoder)//解密
{
	int i;
	int cnt;
	uint8_t txbuf[512];
	uint8_t rxbuf[512];

	int retval;
	int msglen;

	msglen = 5;
	memset(rxbuf, 0, sizeof(rxbuf));
	memset(txbuf, 0, sizeof(txbuf));

	//printf("tx %1d bytes: ", msglen);
	//__android_log_print(ANDROID_LOG_INFO, "SPi", "tx %1d bytes", msglen);
	SM2decoder_CMD[3] = index;

CMD_RESEND:

	memcpy(txbuf, (const void *)SM2decoder_CMD, sizeof(SM2decoder_CMD));

	SendCMD(txbuf, rxbuf);

	RcvINS(txbuf, rxbuf, txbuf[1]);

	SendId(txbuf, rxbuf, 0x55);

	memcpy(txbuf, to_decoder, 128);

	SendData(txbuf, rxbuf, 128);

	SendEnd(txbuf, rxbuf);

	memcpy(txbuf, (const void *)SM2decoder_CMD, sizeof(SM2decoder_CMD));

	RcvINS(txbuf, rxbuf, txbuf[1]);

	RcvLEN(txbuf, rxbuf + 1, 1);

	RcvData(txbuf, rxbuf + 2);

	RcvSW(txbuf, rxbuf + 2 + rxbuf[1], 0x90);

	//__android_log_print(ANDROID_LOG_INFO, "SPi", "rx %1d bytes:", rxbuf[1]+4);
	memcpy(out_decoder, rxbuf + 2, rxbuf[1]-1);

	return 0;
}

int NrsecPort::SM2cert(int type, int index, string cert, uint8_t *out_cert)//证书
{
	int i;
	int cnt;
	uint8_t txbuf[512];
	uint8_t rxbuf[1024];

	int retval;
	int msglen;

	msglen = 5;
	memset(rxbuf, 0, sizeof(rxbuf));
	memset(txbuf, 0, sizeof(txbuf));

	//printf("tx %1d bytes: ", msglen);

	int certlen = cert.length();
	
	SM2cert_CMD[2] = type;
	SM2cert_CMD[3] = index;
	SM2cert_CMD[4] = certlen;

CMD_RESEND:

	memcpy(txbuf, (const void *)SM2cert_CMD, sizeof(SM2cert_CMD));

	SendCMD(txbuf, rxbuf);

	RcvINS(txbuf, rxbuf, txbuf[1]);

	SendId(txbuf, rxbuf, 0x55);

	memcpy(txbuf, cert.c_str(), certlen);

	SendData(txbuf, rxbuf, certlen);

	SendEnd(txbuf, rxbuf);

	memcpy(txbuf, (const void *)SM2cert_CMD, sizeof(SM2cert_CMD));

	RcvINS(txbuf, rxbuf, txbuf[1]);

	RcvLEN(txbuf, rxbuf + 1, 2);

	uint8_t outlen =(*(rxbuf+1)<<8)+(*(rxbuf+2));

	RcvData(txbuf, rxbuf + 3, outlen);

	RcvSW(txbuf, rxbuf + 3 + rxbuf[1], 0x90);

	//__android_log_print(ANDROID_LOG_INFO, "SPi", "rx %1d bytes:", rxbuf[1]+4);

	memcpy(out_cert, rxbuf + 3, outlen-1);

	return 0;
}
int NrsecPort::Indentify(uint8_t *to_idt, uint8_t *out_idt)//安全认证
{
	int i;
	int cnt;
	uint8_t txbuf[512];
	uint8_t rxbuf[512];

	int retval;
	int msglen;

	msglen = 5;
	memset(rxbuf, 0, sizeof(rxbuf));
	memset(txbuf, 0, sizeof(txbuf));

	//printf("tx %1d bytes: ", msglen);
	//__android_log_print(ANDROID_LOG_INFO, "SPi", "tx %1d bytes", msglen);

CMD_RESEND:

	memcpy(txbuf, (const void *)Indentify_CMD, sizeof(Indentify_CMD));

	SendCMD(txbuf, rxbuf);

	RcvINS(txbuf, rxbuf, txbuf[1]);

	SendId(txbuf, rxbuf, 0x55);

	memcpy(txbuf, to_idt, 32);

	SendData(txbuf, rxbuf, 32);

	SendEnd(txbuf, rxbuf);

	memcpy(txbuf, (const void *)SM2decoder_CMD, sizeof(SM2decoder_CMD));

	RcvINS(txbuf, rxbuf, txbuf[1]);

	RcvLEN(txbuf, rxbuf + 1, 1);

	RcvData(txbuf, rxbuf + 2);

	RcvSW(txbuf, rxbuf + 2 + rxbuf[1], 0x90);

	//__android_log_print(ANDROID_LOG_INFO, "SPi", "rx %1d bytes:", rxbuf[1]+4);
	memcpy(out_idt, rxbuf + 2, rxbuf[1]-1);

	return 0;
}
