#ifndef __NRSECPORT_H__
#define __NRSECPORT_H__

#include "SpiPort.h"
#include <mutex>


#define CMD_HEAD_SIZE           5
using namespace std;
typedef uint8_t uint8_t;

class NrsecPort : public SpiPort {
public:

    int SM1ImportKey(const uint8_t* ek, const uint8_t* ak);
    int ImportIV(const uint8_t* iv, uint8_t ivLength);

    int SM1Encrypt(const uint8_t* data, uint16_t dataLen, uint8_t* encryptedData, uint16_t bufferLen);
    int SM1Decrypt(const uint8_t* encryptedData, uint16_t encryptedDataLen, uint8_t* data, uint16_t bufferLen);

	int Random(uint8_t* output, uint8_t length);
	std::string Version();
	int Indentify(uint8_t *to_idt, uint8_t *out_idt);
	int SM2keypair(int index);
	int SM2ExportPublicKey(int index, uint8_t result[]);
	int SM2ExportPrivateKey(int index, uint8_t result[]);
	int SM2ImportPublicKey(int index, const uint8_t new_key[]);
	int SM2ImportPrivateKey(int index, const uint8_t new_key[]);
	int SM3Hash(uint8_t *to_hash, int len, uint8_t *out_hash);
	int SM3Hash(uint8_t *in, int inl, uint8_t *out, uint8_t *pubkey, uint8_t *pucID, int idl);
	int SM2Sign(int index, const uint8_t *to_sign, uint8_t *out_sign);
	int SM2VerifySign(int index, uint8_t *hash, uint8_t * vs);
	int SM2Encrypt(int index, uint8_t *to_encrypt, uint8_t * out_encrypt);
	int SM2Decrypt(int index, uint8_t *to_decoder, uint8_t *out_decoder);
	int SM2cert(int type, int index, string cert, uint8_t *out_cert);

protected:

	uint8_t CalcCRC7(const uint8_t *buff, int len);
	void SendCMD(uint8_t *cmd, uint8_t *rxbuf);
	void RcvINS(uint8_t *txbuf, uint8_t *buf, uint8_t ins);
	void RcvLEN(uint8_t *txbuf, uint8_t *buf, uint8_t len);
	void RcvData(uint8_t *txbuf, uint8_t *buf);
	void RcvData(uint8_t *txbuf, uint8_t *buf, int len);
	void RcvSW(uint8_t *txbuf, uint8_t *buf, uint8_t sw);
	void SendEnd(uint8_t *txbuf, uint8_t *buf);
	void SendId(uint8_t *txbuf, uint8_t *buf, uint8_t id);
	void SendData(uint8_t *data, uint8_t *rxbuf, int data_size);

protected:
	std::mutex m_mutex;

};




#endif // __NRSECPORT_H__
