#include "SpiPort.h"

//typedef unsigned char u8;

#define  RE_SUC   0x01
#define  RE_ERROR 0x00

#define TAG_SPI "SPI"

SpiPort::SpiPort() : m_fd(0)
{
}

SpiPort::~SpiPort()
{
    Close();
}

bool SpiPort::Open(const char *path)
{
    m_path = path;
    m_fd = open(path, O_RDWR);
    if (m_fd < 0) {
        perror("open");
        return false;
    }
    spi_master_init();

    return true;
}

void SpiPort::Close()
{
    if (m_fd > 0)
    {
        close(m_fd);
        m_fd = 0;
    }
}

int SpiPort::spi_transfer(unsigned char *txbuf, unsigned char *rxbuf, int bytes)
{
    int    status;

#if 0
    struct spi_ioc_transfer    xfer[2];
    memset(xfer, 0, sizeof(xfer));
    xfer[0].tx_buf = (__u64)txbuf;
    xfer[0].rx_buf = (__u64)rxbuf;
    xfer[0].len = bytes;
    xfer[0].delay_usecs = 2;

    status = ioctl(m_fd, SPI_IOC_MESSAGE(1), xfer);
#else
    struct spi_ioc_transfer    xfer;
    memset(&xfer, 0, sizeof(xfer));
    xfer.tx_buf = (__u64)txbuf;
    xfer.rx_buf = (__u64)rxbuf;
    xfer.len = bytes;
    xfer.delay_usecs = 2;

    status = ioctl(m_fd, SPI_IOC_MESSAGE(1), &xfer);
#endif
    if (status < 0)
    {
        perror("SPI_IOC_MESSAGE");
        return -1;
    }

    return status;
}

void SpiPort::spi_master_init()
{
    __u8    mode = 3;
    __u8    lsb = 0;
    __u8    bits = 8;
    //__u32    speed = 30000000;
    __u32      speed = 2000000;
    //__u32    speed = 2000000;
    // __u32    speed = 33000000;

    // SPI_IOC_WR_MODE
    int res = 0;

    res = ioctl(m_fd, SPI_IOC_WR_MODE, &mode);
    res = ioctl(m_fd, SPI_IOC_WR_MAX_SPEED_HZ, &speed);
    res = ioctl(m_fd, SPI_IOC_WR_LSB_FIRST, &lsb);

    if (ioctl(m_fd, SPI_IOC_RD_MODE, &mode) < 0)
    {
        perror("SPI rd_mode");
        return;
    }

    if (ioctl(m_fd, SPI_IOC_RD_LSB_FIRST, &lsb) < 0)
    {
        perror("SPI rd_lsb_fist");
        return;
    }

    if (ioctl(m_fd, SPI_IOC_RD_BITS_PER_WORD, &bits) < 0)
    {
        perror("SPI rd bits_per_word");
        return;
    }

    if (ioctl(m_fd, SPI_IOC_RD_MAX_SPEED_HZ, &speed) < 0)
    {
        perror("SPI rd max_speed_hz");
        return;
    }

    __android_log_print(ANDROID_LOG_INFO, TAG_SPI, "%s: spi mode %d, %d bits %sper word, %d Hz max\n", m_path.c_str(), mode, bits, lsb ? "(lsb first) " : "", speed);
    //printf("%s: spi mode %d, %d bits %sper word, %d Hz max\n",
    //    name, mode, bits, lsb ? "(lsb first) " : "", speed);
}

int SpiPort::delay(int x)
{
#ifdef _WIN32
    std::this_thread::sleep_for(std::chrono::milliseconds(x));
#else
    usleep(x);
#endif
    return 0;
}
