package com.xinyingpower.testcomm;

import androidx.appcompat.app.AppCompatActivity;

import android.content.Context;
import android.graphics.ImageFormat;
import android.hardware.camera2.CameraCharacteristics;
import android.hardware.camera2.CameraManager;
import android.hardware.camera2.CameraMetadata;
import android.hardware.camera2.params.StreamConfigurationMap;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.util.Size;
import android.view.View;
import android.widget.TextView;

import com.dev.devapi.api.SysApi;
import com.xinyingpower.testcomm.databinding.ActivityMainBinding;

public class MainActivity extends AppCompatActivity {

    // Used to load the 'testcomm' library on application startup.
    static {
        System.loadLibrary("testcomm");
    }

    private ActivityMainBinding binding;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        binding = ActivityMainBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());

        // Example of a call to a native method
        TextView tv = binding.sampleText;
        // tv.setText(stringFromJNI());

        binding.btnSpi.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                // PrintCameraInfo();

                // SysApi.set12VEnable(true);
                String str = testSpi(0);
                binding.sampleText.setText(str);
            }
        });

        Handler handler = new Handler();
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                binding.btnSpi.performClick();
            }
        };
        handler.postDelayed(runnable, 1000);
    }


    private void PrintCameraInfo() {
        CameraManager cameraManager = (CameraManager) getSystemService(Context.CAMERA_SERVICE);

        CameraCharacteristics cameraCharacteristics = null;

        StringBuilder builder = new StringBuilder();

        try {
            cameraCharacteristics = cameraManager.getCameraCharacteristics("0");
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        if (cameraCharacteristics != null) {

            int[] capabilities = cameraCharacteristics.get(CameraCharacteristics.CONTROL_AF_AVAILABLE_MODES);
            if (capabilities != null) {
                for (int c : capabilities) {
                    builder.append("CONTROL_AF_AVAILABLE_MODES: (" + Integer.toString(c) + ")");
                    if (c == CameraMetadata.CONTROL_AF_MODE_OFF) {
                        builder.append(" CONTROL_AF_MODE_OFF");
                    } else if (c == CameraMetadata.CONTROL_AF_MODE_AUTO) {
                        builder.append(" CONTROL_AF_MODE_AUTO");
                    } else if (c == CameraMetadata.CONTROL_AF_MODE_MACRO) {
                        builder.append(" CONTROL_AF_MODE_MACRO");
                    } else if (c == CameraMetadata.CONTROL_AF_MODE_CONTINUOUS_VIDEO) {
                        builder.append(" CONTROL_AF_MODE_CONTINUOUS_VIDEO");
                    } else if (c == CameraMetadata.CONTROL_AF_MODE_CONTINUOUS_PICTURE) {
                        builder.append(" CONTROL_AF_MODE_CONTINUOUS_PICTURE");
                    } else if (c == CameraMetadata.CONTROL_AF_MODE_EDOF) {
                        builder.append(" CONTROL_AF_MODE_EDOF");
                    }
                    builder.append("\n");
                }
            }

            float[] apertures = cameraCharacteristics.get(CameraCharacteristics.LENS_INFO_AVAILABLE_APERTURES);
            if (apertures != null) {
                if (apertures != null) {
                    for (float f : apertures) {
                        builder.append("LENS_INFO_AVAILABLE_APERTURES: " + Float.toString(f) + "\n");
                    }
                }
            }
            Integer maxAf = cameraCharacteristics.get(CameraCharacteristics.CONTROL_MAX_REGIONS_AF);
            builder.append("CONTROL_MAX_REGIONS_AF: " + Integer.toString(maxAf) + "\n");
            float[] lengths = cameraCharacteristics.get(CameraCharacteristics.LENS_INFO_AVAILABLE_FOCAL_LENGTHS);
            if (lengths != null) {
                for (float f : lengths) {
                    builder.append("LENS_INFO_AVAILABLE_FOCAL_LENGTHS: " + Float.toString(f) + "\n");
                }
            }

            Float dis = cameraCharacteristics.get(CameraCharacteristics.LENS_INFO_MINIMUM_FOCUS_DISTANCE);
            builder.append("LENS_INFO_MINIMUM_FOCUS_DISTANCE: " + ((dis == null) ? "null" : dis.toString()) + "\n");

            StreamConfigurationMap map = cameraCharacteristics.get(CameraCharacteristics.SCALER_STREAM_CONFIGURATION_MAP);
            Size[] sizes = map.getOutputSizes(ImageFormat.RAW_SENSOR);

            for (int i = 0; i < sizes.length; i++) { //遍历所有Size
                Size itemSize = sizes[i];
                // Log.e(TAG, "当前itemSize 宽=" + itemSize.getWidth() + "高=" + itemSize.getHeight());
                builder.append("Available Size: (" + itemSize.getWidth() + "," + itemSize.getHeight() + ")\n");
            }

            Log.i("CAM", builder.toString());
            builder.append("");
        }
    }

    /**
     * A native method that is implemented by the 'testcomm' native library,
     * which is packaged with this application.
     */
    public native String stringFromJNI();

    public native String testSpi(int port);
}